/// Support for interoperating with [JavaScript](https://developer.mozilla.org/en-US/docs/Web/JavaScript) APIs.
@JS()
library ngx_core.js;

import 'dart:typed_data';
import 'package:js/js.dart';

export 'dart:js_util';

/// The global namespace object.
@JS('window')
external dynamic get global;

/// Recursively converts the specified JavaScript [value] to a Dart object.
dynamic dartify(Object value) {
  if (value == null) return null;
  if (value is bool || value is num || value is DateTime || value is String || value is TypedData) return value;
  if (value is List) return value.map(dartify).toList();
  return new Map<String, dynamic>.fromIterable(_getProperties(value), key: (item) => item.first.toString(), value: (item) => dartify(item.last));
}

/// Removes a property from the specified JavaScript [object].
/// Returns a value indicating whether the property was successfully deleted.
@JS('Reflect.deleteProperty')
external bool deleteProperty(Object object, Object propertyKey);

/// Returns a map of the own enumerable properties of the specified JavaScript [object].
Map getProperties(Object object) => new Map.fromIterable(_getProperties(object), key: (item) => item.first, value: (item) => item.last);

/// Returns a list of the own enumerable property `[key, value]` pairs of the specified JavaScript [object].
@JS('Object.entries')
external List<List> _getProperties(Object object);
