part of ngx_core;

/// Sets the page title.
@Directive(
  exportAs: 'pageTitle',
  selector: '[pageTitle]'
)
class PageTitle implements AfterViewInit {

  /// The application properties.
  final Application _app;

  /// The current HTML document.
  final dom.HtmlDocument _document;

  /// The underlying HTML element.
  final dom.Element _element;

  /// Creates a new `pageTitle` directive.
  PageTitle(this._app, @Inject(DOCUMENT) this._document, this._element);

  /// The text of the page title.
  @Input('pageTitle')
  String text = '';

  /// Method called when the component's view has been fully initialized.
  @override
  void ngAfterViewInit() {
    var title = text.isNotEmpty ? text : _element.text.trim().replaceAll(new RegExp(r'\s+'), ' ');
    _document.title = title.isNotEmpty ? '$title – ${_app.name}' : _app.name;
  }
}
