part of ngx_core;

/// Defines the methods and properties of the client application.
@Injectable()
class Application {

  /// The application settings.
  final Configuration _configuration;

  /// The current HTML document.
  final dom.HtmlDocument _document;

  /// The service providing access to the meta tags.
  final Meta _meta;

  /// Creates a new configuration service.
  Application(@Inject(envToken) @Optional() String environment, this._configuration, @Inject(DOCUMENT) this._document, this._meta):
    environment = environment ?? const String.fromEnvironment('env', defaultValue: 'development');

  /// Value indicating whether the application runs in debug mode.
  bool get debug => _configuration['debug'] ?? const ['development', 'test'].contains(environment);

  /// The application environment. Defaults to `"development"`.
  final String environment;

  /// Value indicating whether the application shoud use a secure connection.
  bool get forceSsl => _configuration['forceSsl'] ?? false;

  /// The language that the document is written in.
  String get language => _document.documentElement.lang;
  set language(String value) => _document.documentElement.lang = value;

  /// The application name. Defaults to `"My Application"`.
  String get name => _meta['application-name'] ?? 'My Application';
  set name(String value) => _meta['application-name'] = value;

  /// The base URL of the remote services.
  Uri get serviceUrl => _configuration['serviceUrl'] is String ? Uri.parse(_configuration['serviceUrl']) : null;

  /// The version number of the application package. Defaults to `"1.0"`.
  String get version => _configuration['version'] ?? const String.fromEnvironment('version', defaultValue: '1.0');

  /// Forces the currently requested URL to use a secure channel (e.g. HTTPS).
  void useSecureConnection() {
    var url = dom.window.location;
    if (url.protocol != 'https:') url.assign(url.href.replaceFirst(new RegExp(r'^[^:]+:'), 'https:'));
  }
}
