part of ngx_core;

/// Provides access to the HTML document.
@Injectable()
class DocumentRef {

  /// Creates a new document service.
  DocumentRef(@Inject(DOCUMENT) this.nativeDocument);

  /// The underlying HTML document.
  final dom.HtmlDocument nativeDocument;

  /// Appends a JavaScript file with a given [id] to the document.
  Future appendScript(String id, String src) {
    var script = new dom.ScriptElement()
      ..async = true
      ..id = id;

    var future = _appendElement(script);
    script.src = src;
    return future;
  }

  /// Appends a CSS stylesheet with a given [id] to the document.
  Future appendStylesheet(String id, String href) {
    var stylesheet = new dom.LinkElement()
      ..id = id
      ..rel = 'stylesheet';

    var future = _appendElement(stylesheet);
    stylesheet.href = href;
    return future;
  }

  /// Appends an [element] to the document.
  Future _appendElement(dom.Element element) {
    var completer = new Completer<dom.Event>();
    element
      ..onError.listen(completer.completeError)
      ..onLoad.listen(completer.complete);

    var node = dom.querySelector('#{element.id}');
    if (node == null) nativeDocument.head.append(element);
    else {
      node.parent.insertBefore(element, node);
      node.remove();
    }

    return completer.future;
  }
}
