import 'package:ngx_core/js.dart';
import 'package:test/test.dart';

/// Tests the features of the `ngx_core.js` library.
void main() {
  group('global', () {
    test('should be the global namespace object', () {
      expect(hasProperty(global, 'Array'), isTrue);
      expect(hasProperty(global, 'Boolean'), isTrue);
      expect(hasProperty(global, 'Date'), isTrue);
      expect(hasProperty(global, 'Number'), isTrue);
      expect(hasProperty(global, 'Symbol'), isTrue);
    });

    test('should allow the creation or deletion of new properties', () {
      expect(getProperty(global, 'fooBar'), isNull);
      setProperty(global, 'fooBar', 'bazQux');
      expect(getProperty(global, 'fooBar'), equals('bazQux'));
      deleteProperty(global, 'fooBar');
      expect(getProperty(global, 'fooBar'), isNull);
    });
  });

  group('dartify()', () {
    test('should leave unchanged the supported Dart types', () {
      var now = new DateTime.now();
      expect(dartify(null), isNull);
      expect(dartify(false), isFalse);
      expect(dartify(123), equals(123));
      expect(dartify(-123.456), equals(-123.456));
      expect(dartify(now), equals(now));
      expect(dartify('foo'), equals('foo'));
    });

    test('should convert JavaScript arrays to `List`', () {
      List array = jsify(['foo', 'bar']);
      expect(dartify(array), orderedEquals(<String>['foo', 'bar']));
    });

    test('should convert JavaScript objects to `Map`', () {
      Object object = jsify({'foo': 'bar'});
      expect(dartify(object), allOf(isMap, hasLength(1), containsPair('foo', 'bar')));
    });
  });

  group('deleteProperty()', () {
    test('should remove the property from the specified object', () {
      Object object = jsify({'foo': 'bar'});
      expect(hasProperty(object, 'foo'), isTrue);
      expect(deleteProperty(object, 'foo'), isTrue);
      expect(hasProperty(object, 'foo'), isFalse);
    });
  });

  group('getProperties()', () {
    test('should return the enumerable properties of the specified object', () {
      Object object = jsify({'foo': 'bar'});
      expect(getProperties(object), allOf(hasLength(1), containsPair('foo', 'bar')));

      setProperty(object, 'baz', 'qux');
      expect(getProperties(object), allOf(hasLength(2), containsPair('baz', 'qux')));
    });
  });
}
